"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Services = require("./services");
const _1 = require("./");
const SingleProcessDiscovery_1 = require("./discovery/SingleProcessDiscovery");
const _ = require("lodash");
const Logger_1 = require("./shared/Logger");
const autokeygen_1 = require("./autokeygen");
const Express = require("express");
const http = require("http");
const bodyParser = require("body-parser");
const morgan = require("morgan");
const events_1 = require("events");
const path = require("path");
const url = require("url");
const URI = require("urijs");
const pathMatcher_1 = require("./shared/pathMatcher");
class ServerConfiguration {
}
exports.ServerConfiguration = ServerConfiguration;
class Server extends events_1.EventEmitter {
    constructor(config = {}) {
        super();
        this.version = require('../package.json').version;
        this.config = config;
        if (!this.config.hostname) {
            this.config.hostname = '127.0.0.1';
        }
        if (!this.config.port) {
            this.config.port = 9080;
        }
        this.services = [];
        this.expressApp = Express();
        this.expressApp.use(bodyParser.json());
        this.httpServer = http.createServer(this.expressApp);
        this.discovery = this.config.discovery || new SingleProcessDiscovery_1.SingleProcessDiscovery();
        this.logger = this.config.logger || new Logger_1.FileConsole(this.config.logPath, this.config.logLevel);
        this.expressApp.use(morgan("short", { "stream": { write: (msg) => { this.logger.info(msg.trim()); } } }));
        this.httpServer.on('upgrade', this.upgradeHandler.bind(this));
    }
    get started() {
        return this._started;
    }
    useMiddleware(...args) {
        this.expressApp.use(args);
        return this;
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._started) {
                throw new Error('The server has already been started.');
            }
            const autoKeysWereGenerated = yield autokeygen_1.autoGenerateKeys(path.join(this.config.dataPath, "keys"));
            yield this.injectLogger();
            this.addServiceEndpoints();
            yield this.startServices();
            yield _1.Promisify(this.httpServer.listen.bind(this.httpServer), this.config.port, this.config.hostname);
            yield this.registerServices();
            yield this.injectDiscovery();
            this._started = true;
            this.emit('serverStarted', this);
            if (autoKeysWereGenerated) {
                const usernamePasswordAuthService = this.services.find(s => s instanceof Services.UsernamePasswordAuthService);
                if (usernamePasswordAuthService) {
                    try {
                        yield usernamePasswordAuthService.attemptToRegister('realm-admin', '', true);
                    }
                    catch (err) {
                        this.logger.info('Failed to create realm-admin user', usernamePasswordAuthService);
                    }
                }
            }
        });
    }
    shutdown() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.deregisterServices();
            yield this.stopServices();
            yield _1.Promisify(this.httpServer.close.bind(this.httpServer));
            this._started = false;
            this.emit('serverStopped', this);
            if (!this.config.logger) {
                this.logger.close();
            }
        });
    }
    addServices(...services) {
        services.forEach(s => this.addService(s));
        return this;
    }
    addService(service, config) {
        if (this._started) {
            throw new Error('Services can only be added before starting the server.');
        }
        let instance = typeof (service) === 'string' ?
            new Services[service](config) : service;
        this.services.push(instance);
        return this;
    }
    removeService(service) {
        if (this._started) {
            throw new Error('Services can only be added before starting the server.');
        }
        if (typeof service === 'string') {
            for (let instance of this.services) {
                let serviceName = Reflect.get(instance.constructor, 'serviceName');
                if (serviceName === service) {
                    this.services.splice(this.services.indexOf(instance), 1);
                }
            }
        }
        else {
            const index = this.services.indexOf(service);
            if (index > -1) {
                this.services.splice(index, 1);
            }
        }
        return this;
    }
    upgradeHandler(req, socket, head) {
        const requestPath = url.parse(req.url).path;
        let numberOfMatches = 0;
        for (let service of this.services) {
            let baseRoute = Reflect.get(service.constructor, 'baseRoute');
            if (!baseRoute) {
                continue;
            }
            let upgradeRoutes = Reflect.get(service, 'upgradeRoutes') || [];
            if (upgradeRoutes.length === 0) {
                continue;
            }
            for (let upgradeRoute of upgradeRoutes) {
                let concatenatedServicePath = baseRoute;
                if (upgradeRoute.path !== '/') {
                    concatenatedServicePath = URI.joinPaths(baseRoute, upgradeRoute.path).toString();
                }
                const params = pathMatcher_1.pathMatcher(concatenatedServicePath, requestPath);
                if (params === false) {
                    continue;
                }
                req.params = params;
                numberOfMatches = numberOfMatches + 1;
                let parameterArguments = Reflect.get(service, 'parameterArguments') || [];
                parameterArguments = parameterArguments.filter(p => p.functionName === upgradeRoute.functionName);
                if (parameterArguments.length > 0) {
                    throw new Error('Decorator Annotations for @Uprade are not yet implemented');
                }
                else {
                    try {
                        service[upgradeRoute.functionName](req, socket, head);
                    }
                    catch (err) {
                        if (err instanceof _1.JSONError) {
                            socket.write(`HTTP/1.1 ${err.status} ${err.message}\r\nUpgrade Failed\r\n\r\n`);
                            socket.end();
                        }
                        else {
                            socket.write('HTTP/1.1 500 Internal Server Error\r\nUpgrade Failed\r\n\r\n');
                            socket.end();
                        }
                    }
                }
            }
        }
        if (numberOfMatches === 0) {
            socket.write('HTTP/1.1 404 Not Found\r\nUpgrade Failed\r\n\r\n');
            socket.end();
        }
    }
    injectLogger() {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = this.services.map((service) => {
                const promise = Promise.resolve();
                let functionName = Reflect.get(service, 'unmuteFunctionName');
                if (functionName && service[functionName]) {
                    promise.then(service[functionName](this.logger));
                }
                functionName = Reflect.get(service, 'unmuteContextualFunctionName');
                if (functionName && service[functionName]) {
                    const serviceName = Reflect.get(service.constructor, 'serviceName') || service.constructor.name;
                    promise.then(service[functionName](this.logger.withContext({ service: serviceName })));
                }
                return promise;
            });
            yield Promise.all(promises);
        });
    }
    addServiceEndpoints() {
        for (const service of this.services) {
            let routes = Reflect.get(service, 'routes');
            if (!routes) {
                return;
            }
            let baseRoute = Reflect.get(service.constructor, 'baseRoute');
            if (!baseRoute) {
                throw new Error(`${service} does not have a base route`);
            }
            const router = Express.Router();
            let serviceStaticRoutes = Reflect.get(service, 'serveStaticRoutes') || [];
            for (let serviceStaticRoute of serviceStaticRoutes) {
                router.use(serviceStaticRoute.path, Express.static(serviceStaticRoute.path));
            }
            for (let route of routes) {
                let middlewaresBefore = Reflect.get(service[route.functionName], 'middlewaresBefore');
                if (middlewaresBefore) {
                    router.use(route.path, middlewaresBefore);
                }
                let handler = (req, res, next) => __awaiter(this, void 0, void 0, function* () {
                    let functionName = route.functionName;
                    let parameterArgumentMetaData = Reflect.get(service, 'parameterArguments') || [];
                    parameterArgumentMetaData = parameterArgumentMetaData.filter(x => x.functionName === functionName);
                    try {
                        if (parameterArgumentMetaData.length > 0) {
                            let args = [];
                            for (let p of parameterArgumentMetaData) {
                                switch (p.type) {
                                    case 'body':
                                        args[p.argumentIndex] = p.keyPath ? _.get(req.body, p.keyPath) : req.body;
                                        break;
                                    case 'query':
                                        args[p.argumentIndex] = p.keyPath ? _.get(req.query, p.keyPath) : req.query;
                                        break;
                                    case 'headers':
                                        args[p.argumentIndex] = p.keyPath ? req.headers[p.keyPath.toLowerCase()] : req.headers;
                                        break;
                                    case 'params':
                                        args[p.argumentIndex] = p.keyPath ? _.get(req.params, p.keyPath) : req.params;
                                        break;
                                    case 'request':
                                        args[p.argumentIndex] = req;
                                        break;
                                    case 'response':
                                        args[p.argumentIndex] = res;
                                        break;
                                    case 'next':
                                        args[p.argumentIndex] = next;
                                        break;
                                }
                            }
                            let result = yield Promise.resolve(service[functionName](...args));
                            if (result) {
                                res.json(result);
                            }
                        }
                        else {
                            let result = yield Promise.resolve(service[functionName](req, res, next));
                            if (result) {
                                res.json(result);
                            }
                        }
                    }
                    catch (err) {
                        if (err instanceof _1.JSONError) {
                            res.status(err.status).json(err);
                        }
                        else {
                            this.logger.error(err);
                            res.status(500).send({});
                        }
                    }
                });
                router[route.httpMethod](route.path, handler);
            }
            this.expressApp.use(baseRoute, router);
        }
    }
    startServices() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let s of this.services) {
                const startFunctionName = Reflect.get(s, 'startFunctionName');
                if (startFunctionName && s[startFunctionName]) {
                    yield Promise.resolve(s[startFunctionName]());
                }
            }
        });
    }
    stopServices() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let s of this.services) {
                const stopFunctionName = Reflect.get(s, 'stopFunctionName');
                if (stopFunctionName) {
                    yield Promise.resolve(s[stopFunctionName]());
                }
            }
        });
    }
    registerServices() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let s of this.services) {
                let address = this.httpServer.address();
                yield Promise.resolve(this.discovery.registerService(s, address.address, address.port));
            }
        });
    }
    injectDiscovery() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let s of this.services) {
                let injectDiscoveryPropertyKey = Reflect.get(s, 'injectDiscoveryPropertyKey');
                let injectDiscoveryFunctionName = Reflect.get(s, 'injectDiscoveryFunctionName');
                if (injectDiscoveryPropertyKey) {
                    s[injectDiscoveryPropertyKey] = this.discovery;
                }
                if (injectDiscoveryFunctionName) {
                    yield Promise.resolve(s[injectDiscoveryFunctionName](this.discovery));
                }
            }
        });
    }
    deregisterServices() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let service of this.services) {
                yield this.discovery.deregisterService(service);
            }
        });
    }
}
exports.Server = Server;
//# sourceMappingURL=Server.js.map