"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Server_1 = require("../Server");
const services = require("../services");
const chai_1 = require("chai");
const Realm = require("realm");
const path = require("path");
const faker = require("faker");
const tmp = require("tmp");
const del = require("del");
describe('Realm-JS integration', function () {
    const PORT = 5678;
    const HOST = '127.0.0.1';
    const REALM_JS_ACCESS_TOKEN = 'eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJOb3RpZmllciI6dHJ1ZSwiU3luYyI6dHJ1ZSwiaWF0IjoxNDkyNjExMzEzfQ.Yb-o622ZYLKCQ0kTEDpBm6FsG68X9nj-OCdD4dT1G5W10fjhsTdUJiR1uYBilZPqgZEmIdDgd59rDpEjnqDhBCjTzpbsMVBGQrHj3B5eW4oGvxDkdqjR_fjwc5-zH3BeXMd9UAj13bLrxLsM0_g1FXIjmXCOK7YBrL0kKYAI2IVjqAGsI8zThhmbqxDvBeD8TVPc-brBZvXXHPO63ierJevjtZRz7mg_oV-B13gXXIxy2jlnevqFyBVRF6cd3bJmIpLUXHecQXqDGDeLaVlo2NDcnxl5OoVBN1QUSxptgQc-FIq-5qtxZXHooIJUpKQwtD2mBkJ-HBpxWbQR2C90UA';
    Realm.Sync.setFeatureToken(REALM_JS_ACCESS_TOKEN);
    const ProductSchema = {
        name: 'Product',
        primaryKey: 'productId',
        properties: {
            productId: { type: 'int', optional: false },
            name: { type: 'string', optional: false },
            price: { type: 'float', optional: false }
        }
    };
    let server;
    let tmpDir;
    before(() => __awaiter(this, void 0, void 0, function* () {
        yield del(["./data"]);
        tmpDir = tmp.dirSync();
        Realm.clearTestState();
        server = new Server_1.Server({
            port: PORT,
            logLevel: 'off',
            dataPath: tmpDir.name
        });
        server.addService(new services.UsernamePasswordAuthService({
            privateKeyPath: path.join(server.config.dataPath, "keys/auth.key"),
        }));
        server.addService(new services.RealmManagementService({
            privateKeyPath: path.join(server.config.dataPath, "keys/auth.key"),
        }));
        server.addService(new services.SyncProxyService());
        server.addService(new services.SyncService({
            dataPath: path.join(server.config.dataPath, "sync"),
            publicKeyPath: path.join(server.config.dataPath, "keys/auth.pub"),
        }));
        yield server.start();
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        if (server) {
            yield server.shutdown();
        }
    }));
    it('should be able to register to realm-js', function (done) {
        Realm.Sync.User.register(`http://${HOST}:${PORT}`, 'sampleuser', 'samplepassword', (err, user) => {
            if (err) {
                done(err);
            }
            else {
                chai_1.expect(user).to.not.be.null;
                done();
            }
        });
    });
    it('should be able to login to realm-js', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const registeredUser = yield new Promise((resolve, reject) => {
                Realm.Sync.User.register(`http://${HOST}:${PORT}`, 'newuser', 'samplepassword', (err, user) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(user);
                    }
                });
            });
            registeredUser.logout();
            const loggedInUser = yield new Promise((resolve, reject) => {
                Realm.Sync.User.login(`http://${HOST}:${PORT}`, 'newuser', 'samplepassword', (err, user) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(user);
                    }
                });
            });
        });
    });
    it('should create error when trying to login with an unknown username', function (done) {
        Realm.Sync.User.login(`http://${HOST}:${PORT}`, 'sameuser', 'samplepassword', (err, user) => {
            chai_1.expect(err).not.to.not.be.undefined;
            done();
        });
    });
    it('should create error when trying to register an existing username', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const registeredUser = yield new Promise((resolve, reject) => {
                Realm.Sync.User.register(`http://${HOST}:${PORT}`, 'sameuser', 'samplepassword', (err, user) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(user);
                    }
                });
            });
            registeredUser.logout();
            try {
                const registeredUserAgain = yield new Promise((resolve, reject) => {
                    Realm.Sync.User.register(`http://${HOST}:${PORT}`, 'sameuser', 'samplepassword', (err, user) => {
                        if (err) {
                            reject(err);
                        }
                        else {
                            resolve(user);
                        }
                    });
                });
            }
            catch (err) {
                chai_1.expect(err).to.not.be.undefined;
            }
        });
    });
    function loginAndGetRealm(address, username, password, localPath) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                Realm.Sync.User.login(`http://${address}`, username, password, (err, user) => {
                    if (err) {
                        return reject(err);
                    }
                    const realm = new Realm({
                        path: localPath,
                        sync: {
                            url: `realm://${HOST}:${PORT}/product`,
                            user: user
                        },
                        schema: [ProductSchema]
                    });
                    resolve(realm);
                });
            });
        });
    }
    it('should be able to write to a realm', () => __awaiter(this, void 0, void 0, function* () {
        const localPath1 = path.join(tmpDir.name, "local1.realm");
        const realm1 = yield loginAndGetRealm(`${HOST}:${PORT}`, 'sampleuser', 'samplepassword', localPath1);
        const localPath2 = path.join(tmpDir.name, "local2.realm");
        const realm2 = yield loginAndGetRealm(`${HOST}:${PORT}`, 'sampleuser', 'samplepassword', localPath2);
        return new Promise((resolve, reject) => {
            let obj1;
            realm2.addListener('change', (realm, event) => {
                const obj2 = realm2.objectForPrimaryKey('Product', 0);
                chai_1.assert.equal(obj1.name, obj2.name);
                chai_1.assert.equal(obj1.price, obj2.price);
                resolve();
            });
            realm1.write(() => {
                obj1 = realm1.create('Product', {
                    productId: 0,
                    name: faker.commerce.productName(),
                    price: faker.random.number({ min: 12, max: 7000 })
                }, true);
            });
        });
    }));
});
//# sourceMappingURL=realmjs-integration-tests.spec.js.map